/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <string.h>
#include <stdlib.h>
#include <efl_extension.h>
#include <time.h>

#include "view/create-view.h"
#include "view/recurrence-view.h"
#include "utils/logger.h"
#include "utils/config.h"
#include "utils/event-record.h"
#include "utils/ui-utils.h"
#include "utils/settings-utils.h"

typedef struct
{
    window_obj          *win;
    Evas_Object         *navi;
    Elm_Object_Item     *navi_item;
    Evas_Object         *layout;
    char                *record_title;
    char                *record_location;
    char                *record_description;
    const  char         *btn_reminder_label;
    Evas_Object         *gl_controls;
    struct tm           stm; // start
    struct tm           etm; // end
    Evas_Object         *calendar_btn;
    Evas_Object         *reminder_btn;
    Evas_Object         *priority_btn;
    Evas_Object         *sensitivity_btn;
    Evas_Object         *status_btn;
    Evas_Object         *recurrence_btn;
    Evas_Object         *calendar_type_ctxpopup;
    Evas_Object         *reminder_ctxpopup;
    Evas_Object         *priority_ctxpopup;
    Evas_Object         *sensitivity_ctxpopup;
    Evas_Object         *status_ctxpopup;

    Evas_Object         *popup;
    Evas_Object         *date_time_popup;

    Evas_Object         *start_datetime;
    Evas_Object         *end_datetime;
    Evas_Object         *start_date_btn;
    Evas_Object         *start_time_btn;
    Evas_Object         *end_date_btn;
    Evas_Object         *end_time_btn;

    Eina_Bool           is_all_day;
    calendar_event_priority_e priority;
    calendar_alarm_time_unit_type_e  reminder_type;
    char                *reminder_type_name;
    calendar_sensitivity_e sensitivity;
    calendar_event_status_e status;
    int                 calendar_type;
    int                 reminder;
    Evas_Object         *reminder_entry;
    Elm_Object_Item     *genlist_from_item;
    Elm_Object_Item     *genlist_to_item;
    Elm_Object_Item     *genlist_reminder_item;
    Elm_Object_Item     *genlist_location_item;
    Elm_Object_Item     *genlist_description_item;
    recurrence_repeat_data recurrence_data;
} create_view_data;

static const char *label_calendar        = "Calendar";
static const char *label_title           = "Subject";
static const char *subject_guide         = "Enter the subject";
static const char *label_from            = "From";
static const char *label_to              = "To";
static const char *label_time_zone       = "TimeZone";
static const char *label_all_day         = "All day";
static const char *label_location        = "Location";
static const char *location_guide        = "Enter the location";
static const char *label_description     = "Description";
static const char *description_guide     = "Enter the description";
static const char *label_reminder        = "Reminder";
static const char *label_recurrence      = "Recurrence";
static const char *label_priority        = "Priority";
static const char *label_sensitivity     = "Sensitivity";
static const char *label_status          = "Status";
static const char *button_save           = "Save";
static const char *button_cancel         = "Cancel";
static const char *button_set            = "Set";

static bool close_active_popup(create_view_data *view_data);

/**
 * Callback function used for destroy create-view
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  A strdup'()ed string. The caller will free() it when done.
 */
static void create_view_destroy_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    evas_object_del(ad->start_datetime);
    evas_object_del(ad->end_datetime);
    free(ad->record_title);
    free(ad->record_description);
    free(ad->record_location);
    free(data);
}

static bool _is_title_valid(const char * title)
{
    RETVM_IF(!title, false, "passed string is NULL");

    if (strlen(title) == 0)
    {
        return false;
    }

    while(isspace(*title))
    {
        ++title;
    }

    return *title != '\0';
}

static bool _is_datetime_valid(create_view_data *ad)
{
    if (difftime(mktime(&ad->etm), mktime(&ad->stm)) < 0)
    {
        return false;
    }
    return true;
}

static void _popup_button_ok_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Data NULL");
    evas_object_del(ad->popup);
    ad->popup = NULL;
}

static Evas_Object *_create_warning_popup(void *data, const char *popup_text)
{
    create_view_data *ad = (create_view_data *)data;
    RETVM_IF(!ad, NULL, "Passed NULL input data");

    Evas_Object *popup = elm_popup_add(ad->navi);
    RETVM_IF(!popup, NULL, "Can't create popup");

    Evas_Object *popup_content = elm_label_add(popup);
    elm_object_text_set(popup_content, popup_text);
    elm_object_content_set(popup, popup_content);

    Evas_Object * button_ok = elm_button_add(popup);
    RETVM_IF(!button_ok, NULL, "Cannot add button");
    elm_object_text_set(button_ok, "OK");
    elm_object_part_content_set(popup, "button1", button_ok);
    evas_object_smart_callback_add(button_ok, "clicked", _popup_button_ok_cb, ad);

    evas_object_show(popup);

    return popup;
}

/**
 * Callback function called when the save button is clicked
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _save_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");

    RETM_IF(!ad->record_title, "title string NULL");

    elm_toolbar_item_selected_set(event_info, EINA_FALSE);

    if (!_is_title_valid(ad->record_title))
    {
        ad->popup = _create_warning_popup(ad, WARNING_POPUP_TEXT);
        return;
    }

    if (!_is_datetime_valid(ad))
    {
        ad->popup = _create_warning_popup(ad, WARNING_TIME_POPUP_TEXT);
        return;
    }

    calendar_error_e error = CALENDAR_ERROR_NONE;

    error = calendar_connect();
    RETM_IF(error != CALENDAR_ERROR_NONE, "calendar_connect() is failed(%x)", error);

    calendar_record_h record = create_record_with_presets(ad->calendar_type);

    set_record_title(record, ad->record_title);

    set_record_time(&ad->stm, &ad->etm, record, ad->is_all_day);

    set_record_timezone_city(record);

    set_record_location(record, ad->record_location, CALENDAR_RECORD_NO_COORDINATE, CALENDAR_RECORD_NO_COORDINATE);

    set_record_note(record, ad->record_description);

    set_record_priority(record, ad->priority);

    set_record_sensitivity(record, ad->sensitivity);

    set_record_status(record, ad->status);

    set_record_reminder(record, ad->reminder_type_name);

    set_record_recurrence(record, ad->recurrence_data.id);

    if(ad->recurrence_data.id != CALENDAR_RECURRENCE_NONE)
    {
        set_record_repeat_range_type(record, ad->recurrence_data.range_type);
        set_record_repeat_count(record, ad->recurrence_data.count);
        set_record_repeat_until_time(record, &ad->recurrence_data.tm);
    }

    int record_index = 0;
    error = calendar_db_insert_record(record, &record_index);
    WARN_IF(error != CALENDAR_ERROR_NONE, "calendar_db_insert_record() is failed(%x)", error);

    if(record)
    {
        calendar_record_destroy(record, EINA_TRUE);
    }

    error = calendar_disconnect();
    WARN_IF(error != CALENDAR_ERROR_NONE, "calendar_disconnect() is failed(%x)", error);

    elm_naviframe_item_pop(ad->navi);
}

/**
 * Callback function called when the user enter new title
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _title_entry_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");
    if(ad->record_title != NULL)
    {
        free(ad->record_title);
        ad->record_title = NULL;
    }
    ad->record_title = strdup(elm_entry_entry_get(obj));
}

/**
 * Callback function called when the user enter new location
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _location_entry_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");
    if(ad->record_location != NULL)
    {
        free(ad->record_location);
        ad->record_location = NULL;
    }
    ad->record_location = strdup(elm_entry_entry_get(obj));
}

/**
 * Callback function called when the user enter new description
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _description_entry_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");
    if(ad->record_description != NULL)
    {
        free(ad->record_description);
        ad->record_description = NULL;
    }
    ad->record_description = strdup(elm_entry_entry_get(obj));
}

/**
 * Callback function called when the user selected new label
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _reminder_btn_lable_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)evas_object_data_get(obj, KEY_PRIORITY_DATA);
    RETM_IF(!ad, "Passed NULL input data");

    if(!data)
    {
        ERR("Passed NULL input data");
        evas_object_del(ad->reminder_ctxpopup);
        ad->reminder_ctxpopup = NULL;
        return;
    }

    char * name = (char *)data;
    calendar_alarm_time_unit_type_e reminder = settings_reminder_type_get(name);
    elm_object_text_set(ad->reminder_btn, name);
    evas_object_size_hint_weight_set(ad->reminder_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(ad->reminder_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(ad->reminder_btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
    ad->reminder_type = reminder;
    ad->reminder_type_name = name;

    evas_object_del(ad->reminder_ctxpopup);
    ad->reminder_ctxpopup = NULL;
}

/**
 * Callback function called when the user selected new label
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _recurrence_btn_lable_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");

    switch(ad->recurrence_data.id)
    {
        case CALENDAR_RECURRENCE_NONE:
            elm_object_text_set(ad->recurrence_btn, "None");
            break;
        case CALENDAR_RECURRENCE_DAILY:
            elm_object_text_set(ad->recurrence_btn, "Daily");
            break;
        case CALENDAR_RECURRENCE_WEEKLY:
            elm_object_text_set(ad->recurrence_btn, "Weekly");
            break;
        case CALENDAR_RECURRENCE_MONTHLY:
            elm_object_text_set(ad->recurrence_btn, "Monthly");
            break;
        case CALENDAR_RECURRENCE_YEARLY:
            elm_object_text_set(ad->recurrence_btn, "Yearly");
            break;
        default:
            break;
    }
    evas_object_size_hint_weight_set(ad->recurrence_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(ad->recurrence_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(ad->recurrence_btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
}

/**
 * Callback function called when the user selected new label
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _priority_btn_lable_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");

    create_view_data *ad = (create_view_data *)evas_object_data_get(obj, KEY_PRIORITY_DATA);
    RETM_IF(!ad, "Passed NULL input data");

    evas_object_del(ad->priority_ctxpopup);
    ad->priority_ctxpopup = NULL;

    calendar_event_priority_e prior = (calendar_event_priority_e)data;

    switch(prior)
    {
        case CALENDAR_EVENT_PRIORITY_LOW:
            elm_object_text_set(ad->priority_btn, "Low");
            break;
        case CALENDAR_EVENT_PRIORITY_NORMAL:
            elm_object_text_set(ad->priority_btn, "Normal");
            break;
        case CALENDAR_EVENT_PRIORITY_HIGH:
            elm_object_text_set(ad->priority_btn, "High");
            break;
        default:
            break;
    }
    evas_object_size_hint_weight_set(ad->priority_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(ad->priority_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(ad->priority_btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));

    ad->priority = prior;
}

/**
 * Callback function called when the user selected new label
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _sensitivity_btn_lable_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)evas_object_data_get(obj, KEY_PRIORITY_DATA);
    RETM_IF(!ad, "Passed NULL input data");

    calendar_sensitivity_e sensitivity = (calendar_sensitivity_e)data;

    switch(sensitivity)
    {
        case CALENDAR_SENSITIVITY_PUBLIC:
            elm_object_text_set(ad->sensitivity_btn, "Public");
            break;
        case CALENDAR_SENSITIVITY_PRIVATE:
            elm_object_text_set(ad->sensitivity_btn, "Private");
            break;
        case CALENDAR_SENSITIVITY_CONFIDENTIAL:
            elm_object_text_set(ad->sensitivity_btn, "Confidential");
            break;
        default:
            break;
    }
    evas_object_size_hint_weight_set(ad->sensitivity_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(ad->sensitivity_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(ad->sensitivity_btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
    ad->sensitivity = sensitivity;

    evas_object_del(ad->sensitivity_ctxpopup);
    ad->sensitivity_ctxpopup = NULL;
}

/**
 * Callback function called when the user selected new label
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _status_btn_lable_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)evas_object_data_get(obj, KEY_PRIORITY_DATA);
    RETM_IF(!ad, "Passed NULL input data");

    calendar_event_status_e status = (calendar_event_status_e)data;

    elm_object_text_set(ad->status_btn, settings_status_name_get(status));
    ad->status = status;
    evas_object_size_hint_weight_set(ad->status_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(ad->status_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(ad->status_btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));

    evas_object_del(ad->status_ctxpopup);
    ad->status_ctxpopup = NULL;
}

static void _dismiss_priority_context_popup(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *view_data = (create_view_data *)data;
    RETM_IF(!view_data, "Passed NULL input data");

    if (view_data->priority_ctxpopup)
    {
        evas_object_del(view_data->priority_ctxpopup);
        view_data->priority_ctxpopup = NULL;
    }
}

static void _show_priority_context_popup(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *view_data = (create_view_data *)data;
    RETM_IF(!view_data, "Passed NULL input data");

    if (view_data->priority_ctxpopup)
    {
        evas_object_del(view_data->priority_ctxpopup);
        view_data->priority_ctxpopup = NULL;
    }

    view_data->priority_ctxpopup = elm_ctxpopup_add(view_data->navi);
    evas_object_data_set(view_data->priority_ctxpopup, KEY_PRIORITY_DATA, view_data);

    elm_ctxpopup_item_append(view_data->priority_ctxpopup, "Low", NULL, _priority_btn_lable_changed_cb, (void *)CALENDAR_EVENT_PRIORITY_LOW);
    elm_ctxpopup_item_append(view_data->priority_ctxpopup, "Normal", NULL, _priority_btn_lable_changed_cb, (void *)CALENDAR_EVENT_PRIORITY_NORMAL);
    elm_ctxpopup_item_append(view_data->priority_ctxpopup, "High", NULL, _priority_btn_lable_changed_cb, (void *)CALENDAR_EVENT_PRIORITY_HIGH);

    evas_object_size_hint_weight_set(view_data->priority_ctxpopup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

    int x = 0;
    int y = 0;
    evas_object_geometry_get(view_data->priority_btn, &x, &y, NULL, NULL);
    evas_object_move(view_data->priority_ctxpopup, x, y);
    evas_object_smart_callback_add(view_data->priority_ctxpopup, "dismissed",  _dismiss_priority_context_popup, view_data);

    evas_object_show(view_data->priority_ctxpopup);
}

static void _status_popup_dismiss_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    create_view_data *view_data = (create_view_data *)data;

    if (view_data->status_ctxpopup)
    {
        evas_object_del(view_data->status_ctxpopup);
        view_data->status_ctxpopup = NULL;
    }
}

static void _show_btn_status_context_popup(create_view_data *view)
{
    view->status_ctxpopup = elm_ctxpopup_add(view->navi);
    evas_object_data_set(view->status_ctxpopup, KEY_PRIORITY_DATA, view);

    int i;
    for (i = 0; i < status_info_size; i++)
    {
        elm_ctxpopup_item_append(view->status_ctxpopup, status_info[i].name, NULL, _status_btn_lable_changed_cb, (void *)status_info[i].type);
    }

    evas_object_size_hint_weight_set(view->status_ctxpopup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

    int x = 0;
    int y = 0;
    evas_object_geometry_get(view->status_btn, &x, &y, NULL, NULL);
    evas_object_move(view->status_ctxpopup, x, y);
    evas_object_smart_callback_add(view->status_ctxpopup, "dismissed", _status_popup_dismiss_cb, view);

    evas_object_show(view->status_ctxpopup);
}

static void _show_reminder_context_popup(create_view_data *ad)
{
    ad->reminder_ctxpopup = elm_ctxpopup_add(ad->navi);
    evas_object_data_set(ad->reminder_ctxpopup, KEY_PRIORITY_DATA, ad);

    int i;
    for (i = 0; i < reminder_info_size; i++)
    {
        elm_ctxpopup_item_append(ad->reminder_ctxpopup, reminder_info[i].name, NULL, _reminder_btn_lable_changed_cb, (void *)reminder_info[i].name);
    }

    evas_object_size_hint_weight_set(ad->reminder_ctxpopup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

    int x = 0;
    int y = 0;
    evas_object_geometry_get(ad->reminder_btn, &x, &y, NULL, NULL);
    evas_object_move(ad->reminder_ctxpopup, x, y);
    evas_object_smart_callback_add(ad->reminder_ctxpopup, "dismissed",  _reminder_btn_lable_changed_cb, NULL);

    evas_object_show(ad->reminder_ctxpopup);
}

/**
 * Callback function called when the user press reminder button
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _reminder_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");
    if(!ad->reminder_ctxpopup)
    {
        _show_reminder_context_popup(ad);
    }
    else
    {
        evas_object_del(ad->reminder_ctxpopup);
        ad->reminder_ctxpopup = NULL;
    }
}

/**
 * Callback function called when the user press reminder time-label
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _reminder_lbl_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");
    ad->reminder = atoi(elm_entry_entry_get(ad->reminder_entry));
}

/**
 * Callback function called when the user press recurrence button
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _recurrence_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");
    recurrence_view_add(ad->win, ad->navi, &ad->recurrence_data, _recurrence_btn_lable_changed_cb, ad);
}

static void _dismiss_sensitivity_context_popup(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *view_data = (create_view_data *)data;
    RETM_IF(!view_data, "Passed NULL input data");

    if (view_data->sensitivity_ctxpopup)
    {
        evas_object_del(view_data->sensitivity_ctxpopup);
        view_data->sensitivity_ctxpopup = NULL;
    }
}

static void _show_sensitivity_context_popup(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *view_data = (create_view_data *)data;
    RETM_IF(!view_data, "Passed NULL input data");

    if (view_data->sensitivity_ctxpopup)
    {
        evas_object_del(view_data->sensitivity_ctxpopup);
        view_data->sensitivity_ctxpopup = NULL;
    }

    view_data->sensitivity_ctxpopup = elm_ctxpopup_add(view_data->navi);
    evas_object_data_set(view_data->sensitivity_ctxpopup, KEY_PRIORITY_DATA, view_data);

    elm_ctxpopup_item_append(view_data->sensitivity_ctxpopup, "Public", NULL, _sensitivity_btn_lable_changed_cb, (void *)CALENDAR_SENSITIVITY_PUBLIC);
    elm_ctxpopup_item_append(view_data->sensitivity_ctxpopup, "Private", NULL, _sensitivity_btn_lable_changed_cb, (void *)CALENDAR_SENSITIVITY_PRIVATE);
    elm_ctxpopup_item_append(view_data->sensitivity_ctxpopup, "Confidential", NULL, _sensitivity_btn_lable_changed_cb, (void *)CALENDAR_SENSITIVITY_CONFIDENTIAL);

    evas_object_size_hint_weight_set(view_data->sensitivity_ctxpopup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

    int x = 0;
    int y = 0;
    evas_object_geometry_get(view_data->sensitivity_btn, &x, &y, NULL, NULL);
    evas_object_move(view_data->sensitivity_ctxpopup, x, y);
    evas_object_smart_callback_add(view_data->sensitivity_ctxpopup, "dismissed", _dismiss_sensitivity_context_popup, view_data);

    evas_object_show(view_data->sensitivity_ctxpopup);
}

/**
 * Callback function called when the user press status button
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void _status_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");
    if(!ad->status_ctxpopup)
    {
        _show_btn_status_context_popup(data);
    }
    else
    {
        evas_object_del(ad->status_ctxpopup);
        ad->status_ctxpopup = NULL;
    }
}

/**
 * Add genlist to naviframe
 *
 * @param[in]   data       Data used in this function
 *
 */
static void _genlist_add(create_view_data *data)
{
    data->gl_controls = ui_utils_create_genlist(data->layout);
    elm_genlist_select_mode_set(data->gl_controls, ELM_OBJECT_SELECT_MODE_ALWAYS);
    evas_object_size_hint_weight_set(data->gl_controls, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(data->gl_controls, EVAS_HINT_FILL, EVAS_HINT_FILL);
    elm_object_part_content_set(data->layout, "elm.swallow.genlist.layout", data->gl_controls);
    evas_object_show(data->gl_controls);
}

static void _scroll_genlist_top_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");

    if(ad->genlist_location_item == (Elm_Object_Item *)event_info
       || ad->genlist_description_item == (Elm_Object_Item *)event_info
       || ad->genlist_reminder_item == (Elm_Object_Item *)event_info)
    {
        elm_genlist_item_show((Elm_Object_Item *)event_info, ELM_GENLIST_ITEM_SCROLLTO_MIDDLE);
    }
}

/**
 * Add title entry to create-view layout
 *
 * @param[in]   data       Data used in this function
 * @param[in]   part       The container's part name to set (some might accept
 *                         @c NULL for the default part)
 * @param[in]   func       Func the callback function
 *
 * @return                 An entry object
 */
static Evas_Object *_prepare_entry_box(create_view_data *item_data, const char *name, const char *text, const char *quide, Evas_Smart_Cb func)
{
    create_view_data *data = item_data;
    Evas_Object *box = elm_box_add(data->gl_controls);
    elm_box_horizontal_set(box, EINA_TRUE);
    elm_box_align_set(box, 0.0, 0.5);
    elm_box_padding_set(box, BOX_PADDING, 0);
    evas_object_show(box);

    Evas_Object *label = elm_label_add(box);
    elm_object_text_set(label, name);
    evas_object_show(label);

    Evas_Object *entry = elm_entry_add(box);
    elm_entry_single_line_set(entry, EINA_TRUE);
    elm_entry_scrollable_set(entry, EINA_TRUE);
    elm_scroller_policy_set(entry, ELM_SCROLLER_POLICY_AUTO, ELM_SCROLLER_POLICY_OFF);
    elm_entry_cursor_end_set(entry);
    elm_entry_cnp_mode_set(entry, ELM_CNP_MODE_PLAINTEXT);

    evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, EVAS_HINT_FILL);

    elm_object_text_set(entry, text);
    elm_object_part_text_set(entry, "elm.guide", quide);
    evas_object_smart_callback_add(entry, "changed", func, data);
    evas_object_show(entry);

    elm_box_pack_end(box, label);
    elm_box_pack_end(box, entry);

    return box;
}

/**
 * Add title entry to create-view layout
 *
 * @param[in]   data       Data used in this function
 *
 * @return                 An entry object
 */
static Evas_Object *_entry_add(create_view_data *data, ENTRY_TYPE entry_type)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");

    Evas_Object *box = NULL;
    switch(entry_type)
    {
        case TITLE_ENTRY:
            box = _prepare_entry_box(data, label_title, data->record_title, subject_guide, _title_entry_changed_cb);
            break;
        case LOCATION_ENTRY:
            box = _prepare_entry_box(data, label_location, data->record_location, location_guide, _location_entry_changed_cb);
            break;
        case DESCRIPTION_ENTRY:
            box = _prepare_entry_box(data, label_description, data->record_description, description_guide, _description_entry_changed_cb);
            break;
        default:
            break;
    }

    return box;
}

static char *_item_label_calendar_get(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_calendar);
}

static void _calendar_type_btn_lable_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)evas_object_data_get(obj, KEY_PRIORITY_DATA);
    RETM_IF(!ad, "Passed NULL input data");

    if(!data)
    {
        ERR("Passed NULL input data");
        evas_object_del(ad->calendar_type_ctxpopup);
        ad->calendar_type_ctxpopup = NULL;
        return;
    }

    ad->calendar_type = (int)data;
    elm_object_text_set(ad->calendar_btn, settings_calendar_type_name_get(ad->calendar_type));
    evas_object_size_hint_weight_set(ad->calendar_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(ad->calendar_btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
    evas_object_size_hint_min_set(ad->calendar_btn, ELM_SCALE_SIZE(CALENDAR_BUTTON_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));

    evas_object_del(ad->calendar_type_ctxpopup);
    ad->calendar_type_ctxpopup = NULL;
}

static void _show_calendar_type_context_popup(create_view_data *ad)
{
    ad->calendar_type_ctxpopup = elm_ctxpopup_add(ad->navi);
    evas_object_data_set(ad->calendar_type_ctxpopup, KEY_PRIORITY_DATA, ad);

    int i;
    for (i = 0; i < calendar_type_size; i++)
    {
        elm_ctxpopup_item_append(ad->calendar_type_ctxpopup, calendar_type_info[i].name, NULL, _calendar_type_btn_lable_changed_cb, (void *)calendar_type_info[i].type);
    }

    evas_object_size_hint_weight_set(ad->calendar_type_ctxpopup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

    int x = 0;
    int y = 0;
    evas_object_geometry_get(ad->calendar_btn, &x, &y, NULL, NULL);
    evas_object_move(ad->calendar_type_ctxpopup, x, y);
    evas_object_smart_callback_add(ad->calendar_type_ctxpopup, "dismissed",  _calendar_type_btn_lable_changed_cb, NULL);

    evas_object_show(ad->calendar_type_ctxpopup);
}

static void _calendar_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info)
{
    create_view_data *ad = (create_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");
    if(!ad->calendar_type_ctxpopup)
    {
        _show_calendar_type_context_popup(ad);
    }
    else
    {
        evas_object_del(ad->calendar_type_ctxpopup);
        ad->calendar_type_ctxpopup = NULL;
    }
}

static Evas_Object *__get_content_calendar(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;
    if(strcmp(part, "elm.swallow.end") != 0)
    {
        return NULL;
    }

    data->calendar_btn = elm_button_add(obj);

    elm_object_text_set(data->calendar_btn, settings_calendar_type_name_get( data->calendar_type));
    evas_object_smart_callback_add(data->calendar_btn, "clicked", _calendar_btn_clicked_cb, data);
    evas_object_size_hint_weight_set(data->calendar_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_min_set(data->calendar_btn, ELM_SCALE_SIZE(BUTTON_SCALE_CALENDAR_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));

    return data->calendar_btn;
}

static void _calendar_item_del(void *data, Evas_Object *obj)
{
    create_view_data *ad = (create_view_data *)data;
    ad->calendar_btn = NULL;
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static char *_item_label_from_get(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_from);
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static char *_item_label_to_get(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_to);
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static char *_item_label_timezone_get(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_time_zone);
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static Evas_Object *_get_content(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;

    return _entry_add(data, TITLE_ENTRY);
}

static void _start_date_changed(create_view_data *view, Evas_Object* obj)
{
    elm_datetime_value_get(obj, &view->stm);
    close_active_popup(view);
}

static void _end_date_changed(void *data, Evas_Object* obj)
{
    RETM_IF(!data, "Passed NULL input data");
    create_view_data *ad = (create_view_data *)data;

    elm_datetime_value_get(obj, &ad->etm);
    close_active_popup(ad);
}

static Evas_Object *__get_content_timezone(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;
    if(strcmp(part, "elm.swallow.end") != 0)
    {
        return NULL;
    }

    char time_zone_abbreviation[TIME_STR_LEN] = {'\0'};
    char time_shift[TIME_STR_LEN] = {'\0'};
    char output_text[TIME_STR_LEN] = {'\0'};

    time_t local_time = time(NULL);
    struct tm *tm_localtime = localtime(&local_time);

    strftime(time_zone_abbreviation, sizeof(time_zone_abbreviation), "%Z", tm_localtime);
    strftime(time_shift, sizeof(time_shift), "%z", tm_localtime);

    snprintf(output_text, sizeof(output_text), "%s, %s", time_zone_abbreviation, time_shift);

    Evas_Object *label = elm_label_add(data->layout);
    elm_object_text_set(label, output_text);
    evas_object_size_hint_weight_set(label, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(label, EVAS_HINT_FILL, EVAS_HINT_FILL);

    return label;
}

static void _on_datetime_popup_cancel_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    create_view_data *view = data;
    close_active_popup(view);
}

static void _on_datetime_popup_set_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    create_view_data *view = data;

    Elm_Object_Item *selected_item = elm_genlist_selected_item_get(view->gl_controls);

    if(selected_item == view->genlist_from_item)
    {
        _start_date_changed(view, view->start_datetime);
    }
    else if(selected_item == view->genlist_to_item)
    {
        _end_date_changed(view, view->end_datetime);
    }
    elm_genlist_item_update(selected_item);
}

static Evas_Object *_date_time_popup_add(Evas_Object *parent, create_view_data *view,
                                         Evas_Smart_Cb set_cb, Evas_Smart_Cb cancel_cb)
{
    Evas_Object *popup = elm_popup_add(parent);
    view->date_time_popup = popup;

    elm_popup_orient_set(popup, ELM_POPUP_ORIENT_CENTER);
    evas_object_show(popup);

    Evas_Object *cancel_btn = elm_button_add(popup);
    elm_object_style_set(cancel_btn, "popup");
    elm_object_text_set(cancel_btn, button_cancel);
    elm_object_part_content_set(popup, "button1", cancel_btn);
    evas_object_smart_callback_add(cancel_btn, "clicked", cancel_cb, view);

    Evas_Object *set_btn = elm_button_add(popup);
    elm_object_style_set(set_btn, "popup");
    elm_object_text_set(set_btn, button_set);
    elm_object_part_content_set(popup, "button2", set_btn);
    evas_object_smart_callback_add(set_btn, "clicked", set_cb, view);

    return popup;
}

static void _get_start_datetime(create_view_data *view, Evas_Object *parent, const char *format)
{
    if(!view->start_datetime)
    {
        view->start_datetime = elm_datetime_add(parent);
        evas_object_size_hint_weight_set(view->start_datetime, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(view->start_datetime, EVAS_HINT_FILL, EVAS_HINT_FILL);
    }

    elm_datetime_value_set(view->start_datetime, &view->stm);
    elm_datetime_format_set(view->start_datetime, format);
    evas_object_show(view->start_datetime);
}

static void _get_end_datetime(create_view_data *view, Evas_Object *parent, const char *format)
{
    if(!view->end_datetime)
    {
        view->end_datetime = elm_datetime_add(parent);
        evas_object_size_hint_weight_set(view->end_datetime, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(view->end_datetime, EVAS_HINT_FILL, EVAS_HINT_FILL);
    }

    elm_datetime_value_set(view->end_datetime, &view->etm);
    elm_datetime_format_set(view->end_datetime, format);
    evas_object_show(view->end_datetime);
}

static void _start_date_btn_cb(void *data, Evas_Object* obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    create_view_data *view = (create_view_data *)data;

    Evas_Object *popup = _date_time_popup_add(view->win->win, view, _on_datetime_popup_set_cb, _on_datetime_popup_cancel_cb);
    evas_object_show(popup);

    _get_start_datetime(view, popup, "%d %b %Y");

    elm_object_content_set(popup, view->start_datetime);
}

static void _start_time_btn_cb(void *data, Evas_Object* obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    create_view_data *view = (create_view_data *)data;

    Evas_Object *popup = _date_time_popup_add(view->win->win, view, _on_datetime_popup_set_cb, _on_datetime_popup_cancel_cb);
    evas_object_show(popup);

    _get_start_datetime(view, popup, "%R");

    elm_object_content_set(popup, view->start_datetime);
}

static void _end_date_btn_cb(void *data, Evas_Object* obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    create_view_data *view = (create_view_data *)data;

    Evas_Object *popup = _date_time_popup_add(view->win->win, view, _on_datetime_popup_set_cb, _on_datetime_popup_cancel_cb);
    evas_object_show(popup);

    _get_end_datetime(view, popup, "%d %b %Y");

    elm_object_content_set(popup, view->end_datetime);
}

static void _end_time_btn_cb(void *data, Evas_Object* obj, void *event_info)
{
    RETM_IF(!data, "data is null");
    create_view_data *view = (create_view_data *)data;

    Evas_Object *popup = _date_time_popup_add(view->win->win, view, _on_datetime_popup_set_cb, _on_datetime_popup_cancel_cb);
    evas_object_show(popup);

    _get_end_datetime(view, popup, "%R");

    elm_object_content_set(popup, view->end_datetime);
}

static Evas_Object *__get_content_from(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        Evas_Object *box = elm_box_add(obj);
        elm_box_horizontal_set(box, EINA_TRUE);
        evas_object_show(box);

        data->start_date_btn = elm_button_add(box);
        evas_object_show(data->start_date_btn);
        evas_object_smart_callback_add(data->start_date_btn, "clicked", _start_date_btn_cb, data);

        char buf[DATE_TIME_BUF_LEN] = {0};
        strftime(buf, DATE_TIME_BUF_LEN, "%x", &data->stm);
        elm_object_text_set(data->start_date_btn, buf);

        evas_object_size_hint_weight_set(data->start_date_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_min_set(data->start_date_btn, ELM_SCALE_SIZE(BUTTON_SCALE_DATETIME_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
        elm_box_pack_end(box, data->start_date_btn);

        if(!data->is_all_day)
        {
            data->start_time_btn = elm_button_add(box);
            evas_object_show(data->start_time_btn);
            evas_object_smart_callback_add(data->start_time_btn, "clicked", _start_time_btn_cb, data);

            strftime(buf, DATE_TIME_BUF_LEN, "%R", &data->stm);
            elm_object_text_set(data->start_time_btn, buf);
            evas_object_size_hint_weight_set(data->start_time_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
            evas_object_size_hint_min_set(data->start_time_btn, ELM_SCALE_SIZE(BUTTON_SCALE_DATETIME_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
            elm_box_pack_end(box, data->start_time_btn);
        }

        return box;
    }

    return NULL;
}

static Evas_Object *__get_content_to(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        Evas_Object *box = elm_box_add(obj);
        elm_box_horizontal_set(box, EINA_TRUE);
        evas_object_show(box);

        data->end_date_btn = elm_button_add(box);
        evas_object_show(data->end_date_btn);
        evas_object_smart_callback_add(data->end_date_btn, "clicked", _end_date_btn_cb, data);

        char buf[DATE_TIME_BUF_LEN] = {0};
        strftime(buf, DATE_TIME_BUF_LEN, "%x", &data->etm);
        elm_object_text_set(data->end_date_btn, buf);
        evas_object_size_hint_weight_set(data->end_date_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_min_set(data->end_date_btn, ELM_SCALE_SIZE(BUTTON_SCALE_DATETIME_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
        elm_box_pack_end(box, data->end_date_btn);

        data->end_datetime = elm_datetime_add(data->layout);
        evas_object_size_hint_weight_set(data->end_datetime, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(data->end_datetime, EVAS_HINT_FILL, EVAS_HINT_FILL);

        if(!data->is_all_day)
        {
            data->end_time_btn = elm_button_add(box);
            evas_object_show(data->end_time_btn);
            evas_object_smart_callback_add(data->end_time_btn, "clicked", _end_time_btn_cb, data);

            strftime(buf, DATE_TIME_BUF_LEN, "%R", &data->etm);
            elm_object_text_set(data->end_time_btn, buf);
            evas_object_size_hint_weight_set(data->end_time_btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
            evas_object_size_hint_min_set(data->end_time_btn, ELM_SCALE_SIZE(BUTTON_SCALE_DATETIME_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
            elm_box_pack_end(box, data->end_time_btn);
        }

        return box;
    }

    return NULL;
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Allday label
 */
static char *_item_label_allday_get(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_all_day);
}

/**
 * Callback function called when the allday changed
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 */
static void __allday_changed(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    create_view_data *ad = (create_view_data *)data;

    ad->is_all_day = !ad->is_all_day;

    elm_genlist_item_update(ad->genlist_from_item);
    elm_genlist_item_update(ad->genlist_to_item);

    _start_date_changed(ad, ad->start_datetime);
    _end_date_changed(ad, ad->end_datetime);
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Checkbox object
 */
static Evas_Object *__get_content_allday(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;

    if(strcmp(part, "elm.swallow.end") != 0)
    {
        return NULL;
    }

    Eina_Bool allday = EINA_FALSE;
    Evas_Object *check_obj = elm_check_add(data->layout);
    elm_check_state_set(check_obj, allday);
    elm_object_style_set(check_obj, "on&off");
    evas_object_smart_callback_add(check_obj, "changed", __allday_changed, data);
    elm_check_state_set(check_obj, allday);
    evas_object_propagate_events_set(check_obj, EINA_FALSE);

    return check_obj;
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Entry object
 */
static Evas_Object *__get_content_location(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;

    return _entry_add(data, LOCATION_ENTRY);
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Entry object
 */
static Evas_Object *__get_content_description(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;

    return _entry_add(data, DESCRIPTION_ENTRY);
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Reminder label
 */
static char *_item_label_reminder_get(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_reminder);
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Entry or button objects
 */
static Evas_Object *__get_content_reminder(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        Evas_Object *btn = elm_button_add(data->layout);
        data->btn_reminder_label = data->reminder_type_name;
        evas_object_smart_callback_add(btn, "clicked", _reminder_btn_clicked_cb, data);
        elm_object_text_set(btn, data->btn_reminder_label);
        data->reminder_btn = btn;
        evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
        evas_object_size_hint_min_set(btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
        return btn;
    }

    return NULL;
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Recurrence label
 */
static char *_item_label_recurrence_get(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_recurrence);
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Button object
 */
static Evas_Object *__get_content_recurrence(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        Evas_Object *btn = elm_button_add(data->layout);
        elm_object_text_set(btn, "None");
        evas_object_smart_callback_add(btn, "clicked", _recurrence_btn_clicked_cb, data);
        data->recurrence_btn = btn;
        evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
        evas_object_size_hint_min_set(btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
        return btn;
    }

    return NULL;
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Priority label
 */
static char *_item_label_priority_get(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_priority);
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Button object
 */
static Evas_Object *__get_content_priority(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        Evas_Object *btn = elm_button_add(data->layout);
        evas_object_smart_callback_add(btn, "clicked", _show_priority_context_popup, data);
        data->priority_btn = btn;
        switch(data->priority)
        {
            case CALENDAR_EVENT_PRIORITY_LOW:
                elm_object_text_set(data->priority_btn, "Low");
                break;
            case CALENDAR_EVENT_PRIORITY_NORMAL:
                elm_object_text_set(data->priority_btn, "Normal");
                break;
            case CALENDAR_EVENT_PRIORITY_HIGH:
                elm_object_text_set(data->priority_btn, "High");
                break;
            default:
                break;
        }
        evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
        evas_object_size_hint_min_set(btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));

        return btn;
    }

    return NULL;
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Sensitivity label
 */
static char *_item_label_sensitivity_get(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_sensitivity);
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Button object
 */
static Evas_Object *__get_content_sensivity(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        Evas_Object *btn = elm_button_add(data->layout);
        evas_object_smart_callback_add(btn, "clicked", _show_sensitivity_context_popup, data);

        data->sensitivity_btn = btn;

        switch(data->sensitivity)
        {
            case CALENDAR_SENSITIVITY_PUBLIC:
                elm_object_text_set(data->sensitivity_btn, "Public");
                break;
            case CALENDAR_SENSITIVITY_PRIVATE:
                elm_object_text_set(data->sensitivity_btn, "Private");
                break;
            case CALENDAR_SENSITIVITY_CONFIDENTIAL:
                elm_object_text_set(data->sensitivity_btn, "Confidential");
                break;
        }
        evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
        evas_object_size_hint_min_set(btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
        return btn;
    }

    return NULL;
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Status label
 */
static char *_item_label_status_get(void *data, Evas_Object *obj, const char *part)
{
    return strdup(label_status);
}

/**
 * Callback function called when the genlist item creates
 *
 * @param[in]  data         Data used in this function
 * @param[in]  obj          Evas_Object which emit this event
 * @param[in]  event_info   Information about such event
 *
 * @return                  Button object
 */
static Evas_Object *__get_content_status(void *item_data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!item_data, NULL, "Passed NULL input data");
    create_view_data *data = (create_view_data *)item_data;

    if(strcmp(part, "elm.swallow.end") == 0)
    {
        Evas_Object *btn = elm_button_add(data->layout);
        evas_object_smart_callback_add(btn, "clicked", _status_btn_clicked_cb, data);

        data->status_btn = btn;
        elm_object_text_set(data->status_btn, settings_status_name_get(data->status));
        evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
        evas_object_size_hint_align_set(btn, EVAS_HINT_FILL, EVAS_HINT_FILL);
        evas_object_size_hint_min_set(btn, ELM_SCALE_SIZE(BUTTON_SCALE_WIDTH), ELM_SCALE_SIZE(BUTTON_SCALE_HEIGHT));
        return btn;
    }

    return NULL;
}

static Elm_Genlist_Item_Class itc_calendar_type = {
    .item_style = "default",
    .func.text_get = _item_label_calendar_get,
    .func.content_get = __get_content_calendar,
    .func.del = _calendar_item_del
};

static Elm_Genlist_Item_Class itc_title = {
    .item_style = "1icon",
    .func.content_get = _get_content,
};

static Elm_Genlist_Item_Class itc_from = {
    .item_style = "default",
    .func.text_get = _item_label_from_get,
    .func.content_get = __get_content_from,
};

static Elm_Genlist_Item_Class itc_to = {
    .item_style = "default",
    .func.text_get = _item_label_to_get,
    .func.content_get = __get_content_to,
};

static Elm_Genlist_Item_Class itc_timezone = {
    .item_style = "default",
    .func.text_get = _item_label_timezone_get,
    .func.content_get = __get_content_timezone,
};

static Elm_Genlist_Item_Class itc_allday = {
    .item_style = "default",
    .func.text_get = _item_label_allday_get,
    .func.content_get = __get_content_allday,
};

static Elm_Genlist_Item_Class itc_location = {
    .item_style = "1icon",
    .func.content_get = __get_content_location,
};

static Elm_Genlist_Item_Class itc_description = {
    .item_style = "1icon",
    .func.content_get = __get_content_description,
};

static Elm_Genlist_Item_Class itc_reminder = {
    .item_style = "default",
    .func.text_get = _item_label_reminder_get,
    .func.content_get = __get_content_reminder,
};

static Elm_Genlist_Item_Class itc_recurrence = {
    .item_style = "default",
    .func.text_get = _item_label_recurrence_get,
    .func.content_get = __get_content_recurrence,
};

static Elm_Genlist_Item_Class itc_priority = {
    .item_style = "default",
    .func.text_get = _item_label_priority_get,
    .func.content_get = __get_content_priority,
};

static Elm_Genlist_Item_Class itc_sensivity = {
    .item_style = "default",
    .func.text_get = _item_label_sensitivity_get,
    .func.content_get = __get_content_sensivity,
};

static Elm_Genlist_Item_Class itc_status = {
    .item_style = "default",
    .func.text_get = _item_label_status_get,
    .func.content_get = __get_content_status,
};

/**
 * Add items to genlist
 *
 * @param[in]   data       Data used in this function
 *
 */
static void _genlist_items_append(create_view_data *data)
{
    elm_genlist_item_append(data->gl_controls, &itc_calendar_type,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_title,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    data->genlist_from_item =
    elm_genlist_item_append(data->gl_controls, &itc_from,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    data->genlist_to_item =
    elm_genlist_item_append(data->gl_controls, &itc_to,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_timezone,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_allday,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    data->genlist_location_item =
    elm_genlist_item_append(data->gl_controls, &itc_location,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    data->genlist_description_item =
    elm_genlist_item_append(data->gl_controls, &itc_description,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    data->genlist_reminder_item =
    elm_genlist_item_append(data->gl_controls, &itc_reminder,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_recurrence,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_priority,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_sensivity,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_status,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    evas_object_smart_callback_add(data->gl_controls, "selected", _scroll_genlist_top_cb, data);
}

static bool close_active_popup(create_view_data *view_data)
{
    RETVM_IF(!view_data, false, "view_data is NULL");

    if (view_data->calendar_type_ctxpopup)
    {
        evas_object_del(view_data->calendar_type_ctxpopup);
        view_data->calendar_type_ctxpopup = NULL;
    }
    else if (view_data->reminder_ctxpopup)
    {
        evas_object_del(view_data->reminder_ctxpopup);
        view_data->reminder_ctxpopup = NULL;
    }
    else if (view_data->sensitivity_ctxpopup)
    {
        evas_object_del(view_data->sensitivity_ctxpopup);
        view_data->sensitivity_ctxpopup = NULL;
    }
    else if (view_data->priority_ctxpopup)
    {
        evas_object_del(view_data->priority_ctxpopup);
        view_data->priority_ctxpopup = NULL;
    }
    else if (view_data->status_ctxpopup)
    {
        evas_object_del(view_data->status_ctxpopup);
        view_data->status_ctxpopup = NULL;
    }
    else if (view_data->popup)
    {
        evas_object_del(view_data->popup);
        view_data->popup = NULL;
    }
    else if (view_data->date_time_popup)
    {
        Evas_Object *obj = elm_object_content_get(view_data->date_time_popup);
        elm_object_content_unset(view_data->date_time_popup);
        evas_object_hide(obj);
        evas_object_del(view_data->date_time_popup);
        view_data->date_time_popup = NULL;
    }
    else
    {
        return false;
    }
    return true;
}

static Eina_Bool _create_view_navi_pop_cb(void *data, Elm_Object_Item *it)
{
    RETVM_IF(!data, EINA_FALSE, "Passed NULL input data");
    create_view_data *app = (create_view_data *)data;
    if (close_active_popup(app))
    {
        return EINA_FALSE;
    }
    else
    {
        evas_object_del(app->gl_controls);
        app->gl_controls = NULL;
        return EINA_TRUE;
    }
}

/**
 * Add Create view
 *
 * @param[in]   win       Main window object
 * @param[in]   parent    Naviframe object
 * @return      created layout or otherwise NULL
 *
 */
Evas_Object *create_view_add(window_obj *win, Evas_Object *parent)
{
    RETVM_IF(!win || !parent, NULL, "Passed NULL input data");

    create_view_data *data = calloc(1, sizeof(create_view_data));
    RETVM_IF(!data, NULL, "Cannot allocate memory");

    time_t time_now = time(0);

    data->win = win;
    data->navi = parent;
    data->is_all_day = EINA_FALSE;
    data->calendar_type = DEFAULT_EVENT_CALENDAR_BOOK_ID;
    data->priority = CALENDAR_EVENT_PRIORITY_LOW;
    data->sensitivity = CALENDAR_SENSITIVITY_PUBLIC;
    data->status = CALENDAR_EVENT_STATUS_NONE;
    data->recurrence_data.id = CALENDAR_RECURRENCE_NONE;
    data->recurrence_data.range_type = CALENDAR_RANGE_NONE;
    data->layout = ui_utils_layout_add(data->navi, create_view_destroy_cb, data);
    data->reminder_type = CALENDAR_ALARM_TIME_UNIT_MINUTE;
    data->reminder_type_name = "None";
;
    struct tm *loc_tm = localtime(&time_now);
    if(loc_tm)
    {
        data->stm = *loc_tm;
        data->etm = *loc_tm;
    }

    evas_object_size_hint_weight_set(data->layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(data->layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
    elm_layout_file_set(data->layout, GET_PATH(EDJ_CREATEVIEW), "create_view_layout");

    /*Header for naviframe*/
    data->navi_item = elm_naviframe_item_push(data->navi, "Add event", NULL, NULL, data->layout, NULL);
    elm_naviframe_item_pop_cb_set(data->navi_item, _create_view_navi_pop_cb, data);
    evas_object_show(data->layout);

    /*Footer for naviframe*/
    Evas_Object *toolbar = ui_utils_toolbar_add(data->navi, data->navi_item);
    elm_toolbar_item_append(toolbar, NULL, button_save, _save_btn_clicked_cb, data);

    /*Create genlist for Events control*/
    _genlist_add(data);

    _genlist_items_append(data);

    return data->layout;
}
