/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <calendar.h>
#include <stdlib.h>

#include "view/edit-view.h"
#include "view/details-view.h"
#include "utils/logger.h"
#include "utils/config.h"
#include "utils/event-record.h"
#include "utils/config.h"
#include "utils/ui-utils.h"
#include "utils/settings-utils.h"

#define TAB_MULTYPLIER 0.3
#define LABEL_TEXT_SIZE 25
#define DATA_TEXT_SIZE 36

typedef struct
{
    window_obj          *win;
    Evas_Object         *navi;
    Elm_Object_Item     *navi_item;
    Evas_Object         *layout;
    Evas_Object         *gl_controls;
    calendar_record_h   event_record;
    calendar_record_h   alarm_record;
    Evas_Object         *popup;
    Elm_Object_Item     *edit_button;
    Elm_Object_Item     *delete_button;

    int                 event_record_index;
} details_view_data;

static const char *label_subject         = "Subject";
static const char *label_start           = "Start";
static const char *label_due             = "Due";
static const char *label_time_zone       = "TimeZone";
static const char *label_location        = "Location";
static const char *label_description     = "Description";
static const char *label_reminder        = "Reminder";
static const char *label_recurrence      = "Recurrence";
static const char *label_priority        = "Priority";
static const char *label_sensivity       = "Sensitivity";
static const char *label_status          = "Status";
static const char *button_edit           = "Edit";
static const char *button_delete         = "Delete";
static const char *button_cancel         = "Cancel";
static const char *button_ok             = "Ok";
static const char *label_none            = "None";
static const char *label_del_confirm     = "Are you sure you want to delete?";

static const char *label_yes             = "Yes";
static const char *label_no              = "No";
static const char *label_low             = "Low";
static const char *label_normal          = "Normal";
static const char *label_high            = "High";
static const char *label_public          = "Public";
static const char *label_private         = "Private";
static const char *label_confidential    = "Confidential";

static void details_view_destroy_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
    details_view_data *ad = (details_view_data *)data;
    RETM_IF(!ad, "Passed NULL input data");

    if (ad->alarm_record)
    {
        calendar_record_remove_child_record(ad->event_record, _calendar_event.calendar_alarm, ad->alarm_record);
    }

    calendar_record_destroy(ad->event_record, EINA_TRUE);
    free(ad);
}

static void _genlist_add(details_view_data *data)
{
    data->gl_controls = ui_utils_create_genlist(data->layout);
    elm_genlist_select_mode_set(data->gl_controls, ELM_OBJECT_SELECT_MODE_NONE);
    evas_object_size_hint_weight_set(data->gl_controls, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(data->gl_controls, EVAS_HINT_FILL, EVAS_HINT_FILL);
    elm_object_part_content_set(data->layout, "elm.swallow.genlist.layout", data->gl_controls);
    elm_win_resize_object_add(data->win->win, data->gl_controls);
    evas_object_show(data->gl_controls);
}

static Evas_Object *_content_get(void *data, Evas_Object *obj, const char * labelText, const char *data_text)
{
    details_view_data *details_data = (details_view_data *)data;
    int width = 0;
    int heigth = 0;
    char buffer[STRING_SIZE + SUBJECT_CHAR_LIMIT] = {'\0'};

    Evas_Object *box = elm_box_add(obj);
    elm_win_screen_size_get(details_data->win->win, NULL, NULL, &width, NULL);
    elm_box_horizontal_set(box, EINA_TRUE);
    Evas_Object *subject_header_label = elm_label_add(box);
    Evas_Object *label = elm_label_add(box);
    evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);

    int newSize = ELM_SCALE_SIZE(LABEL_TEXT_SIZE);
    snprintf(buffer, sizeof(buffer), "<font_size=%d>%s</font_size>", newSize, labelText);
    elm_object_part_text_set(subject_header_label, "elm.text", buffer);
    snprintf(buffer, sizeof(buffer), "<font_size=%d><font_weight=Bold>%s</font_weight></font_size>", DATA_TEXT_SIZE, data_text);
    elm_object_part_text_set(label, "elm.text", buffer);

    Evas_Object * scroller = elm_scroller_add(box);
    elm_scroller_bounce_set(scroller, EINA_TRUE, EINA_FALSE);
    elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_AUTO, ELM_SCROLLER_POLICY_OFF);
    evas_object_size_hint_weight_set(scroller, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(scroller, EVAS_HINT_FILL, EVAS_HINT_FILL);

    Evas_Object *scroll_box = elm_box_add(scroller);
    elm_box_horizontal_set(scroll_box, EINA_TRUE);
    evas_object_size_hint_weight_set(scroll_box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(scroll_box, EVAS_HINT_FILL, EVAS_HINT_FILL);
    elm_box_align_set(scroll_box, 0.0, 0.5);
    elm_object_content_set(scroller, scroll_box);
    elm_box_pack_end(scroll_box, label);

    elm_box_pack_end(box, subject_header_label);
    elm_box_pack_end(box, scroller);

    evas_object_size_hint_min_get(subject_header_label, NULL, &heigth);
    evas_object_size_hint_min_set(subject_header_label, width * TAB_MULTYPLIER, heigth);
    evas_object_size_hint_max_set(subject_header_label, width * TAB_MULTYPLIER, heigth);
    elm_box_align_set(box, 0.0, 0.5);
    evas_object_show(subject_header_label);
    evas_object_show(label);
    evas_object_show(scroller);
    evas_object_show(box);

    return box;
}

static char *_item_label_get(void *data)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    details_view_data *details_data = (details_view_data *)data;
    char *title = NULL;

    if(CALENDAR_ERROR_NONE != calendar_record_get_str(details_data->event_record, _calendar_event.summary, &title))
    {
        return strdup(label_none);
    }

    return title;

}

static char *_item_label_start_get(void *data)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    details_view_data *details_data = (details_view_data *)data;
    calendar_time_s start_time;
    memset(&start_time, 0x0, sizeof(start_time));
    char *label = calloc(STRING_SIZE, sizeof(char));

    if(CALENDAR_ERROR_NONE != calendar_record_get_caltime(details_data->event_record, _calendar_event.start_time, &start_time))
    {
        return strdup(label_none);
    }

    if(start_time.type == CALENDAR_TIME_UTIME)
    {
        const time_t date = start_time.time.utime;
        struct tm *loc_time = localtime(&date);

        strftime (label , STRING_SIZE, "%x %R", loc_time);
    }
    else
    {
        snprintf(label, STRING_SIZE, "%d/%d/%d ", start_time.time.date.mday, start_time.time.date.month, start_time.time.date.year);
    }

    return label;
}

static char *_item_label_due_get(void *data)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    details_view_data *details_data = (details_view_data *)data;
    calendar_time_s end_time;
    memset(&end_time, 0x0, sizeof(end_time));
    char *label = calloc(STRING_SIZE, sizeof(char));

    if(CALENDAR_ERROR_NONE != calendar_record_get_caltime(details_data->event_record, _calendar_event.end_time, &end_time))
    {
        return strdup(label_none);
    }
    if(end_time.type == CALENDAR_TIME_UTIME)
    {
        const time_t date = end_time.time.utime;
        const struct tm *loc_time = localtime(&date);

        strftime ( label , STRING_SIZE, "%x %R", loc_time);
    }
    else
    {
        snprintf(label, STRING_SIZE, "%d/%d/%d ", end_time.time.date.mday, end_time.time.date.month, end_time.time.date.year);
    }

    return label;
}

static char *_item_label_timezone_get(void *data)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    details_view_data *details_data = (details_view_data *)data;
    char *timezone_path = NULL;

    if(CALENDAR_ERROR_NONE != calendar_record_get_str(details_data->event_record, _calendar_event.start_tzid, &timezone_path))
    {
        return strdup(label_none);
    }

    return timezone_path;
}

static char *_item_label_location_get(void *data)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    details_view_data *details_data = (details_view_data *)data;
    char *location = NULL;

    if(CALENDAR_ERROR_NONE != calendar_record_get_str(details_data->event_record, _calendar_event.location, &location) || !location)
    {
        location = strdup("");
    }

    return location;
}

static char *_item_label_description_get(void *data)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    details_view_data *details_data = (details_view_data *)data;
    char *description = NULL;

    if(CALENDAR_ERROR_NONE != calendar_record_get_str(details_data->event_record, _calendar_event.description, &description) || !description)
    {
        description = strdup("");
    }

    return description;
}

static char *_item_label_reminder_get(void *data)
{
    details_view_data *details_data = (details_view_data *)data;
    int reminder = -1;

    get_record_reminder(details_data->event_record, &reminder, NULL);

    return reminder == -1 ? strdup(label_no) : strdup(label_yes);
}

static char *_item_label_recurrence_get(void *data)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");
    details_view_data *details_data = (details_view_data *)data;
    int freq = CALENDAR_RECURRENCE_NONE;
    char *id_str = NULL;
    const char *label = NULL;

    if(CALENDAR_ERROR_NONE == calendar_record_get_str(details_data->event_record, _calendar_event.recurrence_id, &id_str))
    {
        if (id_str)
        {
            freq = atoi(id_str);
            free(id_str);
        }
        else
        {
            calendar_record_get_int(details_data->event_record, _calendar_event.freq, &freq);
        }
    }

    switch(freq)
    {
        case CALENDAR_RECURRENCE_DAILY:
            label = "Daily";
            break;
        case CALENDAR_RECURRENCE_WEEKLY:
            label = "Weekly";
            break;
        case CALENDAR_RECURRENCE_MONTHLY:
            label = "Monthly";
            break;
        case CALENDAR_RECURRENCE_YEARLY:
            label = "Yearly";
            break;
        case CALENDAR_RECURRENCE_NONE:
        default:
            label = label_none;
            break;
    }

    if(label)
    {
        return strdup(label);
    }
    return NULL;
}

static char *_item_label_priority_get(void *data)
{
    details_view_data *details_data = (details_view_data *)data;
    calendar_error_e error = CALENDAR_ERROR_NONE;
    int priority = 0;
    const char *label = NULL;

    error = calendar_record_get_int(details_data->event_record, _calendar_event.priority, &priority);
    if(error!=CALENDAR_ERROR_NONE || priority == 0)
    {
        label = label_normal;
    }

    switch(priority)
    {
        case CALENDAR_EVENT_PRIORITY_LOW:
            label = label_low;
            break;
        case CALENDAR_EVENT_PRIORITY_NORMAL:
            label = label_normal;
            break;
        case CALENDAR_EVENT_PRIORITY_HIGH:
        default:
            label = label_high;
            break;
    }

    return strdup(label);
}

static char *_item_label_sensivity_get(void *data)
{
    details_view_data *details_data = (details_view_data *)data;
    calendar_error_e error = CALENDAR_ERROR_NONE;
    int sensitivity = 0;
    const char *label = NULL;

    error = calendar_record_get_int(details_data->event_record, _calendar_event.sensitivity, &sensitivity);
    if(error != CALENDAR_ERROR_NONE || sensitivity == 0)
    {
        label = label_public;
    }

    switch(sensitivity)
    {
        case CALENDAR_SENSITIVITY_PUBLIC:
            label = label_public;
            break;
        case CALENDAR_SENSITIVITY_PRIVATE:
            label = label_private;
            break;
        case CALENDAR_SENSITIVITY_CONFIDENTIAL:
        default:
            label = label_confidential;
            break;
    }

    return strdup(label);
}

static char *_item_label_status_get(void *data)
{
    details_view_data *details_data = (details_view_data *)data;
    calendar_error_e error = CALENDAR_ERROR_NONE;
    int status = 0;
    const char *label = NULL;

    error = calendar_record_get_int(details_data->event_record, _calendar_event.event_status, &status);
    if(error)
    {
        label = label_none;
    }
    else
    {
        label = settings_status_name_get(status);
    }

    return strdup(label);
}


static Evas_Object *_item_content_get(void *data, Evas_Object *obj, const char *part)
{
    if(strcmp(part, "elm.icon") != 0)
    {
        return NULL;
    }
    char * data_text = _item_label_get(data);
    Evas_Object *content = _content_get(data, obj, label_subject, data_text);
    if(data_text)
    {
        free(data_text);
    }
    return content;
}

static Evas_Object *_item_content_start_get(void *data, Evas_Object *obj, const char *part)
{
    if(strcmp(part, "elm.icon") != 0)
    {
        return NULL;
    }
    char * data_text = _item_label_start_get(data);
    Evas_Object *content = _content_get(data, obj, label_start, data_text);
    if(data_text)
    {
        free(data_text);
    }
    return content;
}

static Evas_Object *_item_content_due_get(void *data, Evas_Object *obj, const char *part)
{
    if(strcmp(part, "elm.icon") != 0)
    {
        return NULL;
    }
    char * data_text = _item_label_due_get(data);
    Evas_Object *content = _content_get(data, obj, label_due, data_text);
    if(data_text)
    {
        free(data_text);
    }
    return content;
}

static Evas_Object *_item_content_timezone_get(void *data, Evas_Object *obj, const char *part)
{
    if(strcmp(part, "elm.icon") != 0)
    {
        return NULL;
    }
    char * data_text = _item_label_timezone_get(data);
    Evas_Object *content = _content_get(data, obj, label_time_zone, data_text);
    if(data_text)
    {
        free(data_text);
    }
    return content;
}

static Evas_Object *_item_content_location_get(void *data, Evas_Object *obj, const char *part)
{
    if(strcmp(part, "elm.icon") != 0)
    {
        return NULL;
    }
    char * data_text = _item_label_location_get(data);

    Evas_Object *content = _content_get(data, obj, label_location, data_text);
    if(data_text)
    {
        free(data_text);
    }
    return content;
}

static Evas_Object *_item_content_description_get(void *data, Evas_Object *obj, const char *part)
{
    if(strcmp(part, "elm.icon") != 0)
    {
        return NULL;
    }
    char * data_text = _item_label_description_get(data);
    Evas_Object *content = _content_get(data, obj, label_description, data_text);
    if(data_text)
    {
        free(data_text);
    }
    return content;
}

static Evas_Object *_item_content_reminder_get(void *data, Evas_Object *obj, const char *part)
{
    if(strcmp(part, "elm.icon") != 0)
    {
        return NULL;
    }
    char * data_text = _item_label_reminder_get(data);

    Evas_Object *content = _content_get(data, obj, label_reminder, data_text);
    if(data_text)
    {
        free(data_text);
    }
    return content;
}

static Evas_Object *_item_content_recurrence_get(void *data, Evas_Object *obj, const char *part)
{
    RETVM_IF(!data, NULL, "Passed NULL input data");

    if(strcmp(part, "elm.icon") != 0)
    {
        return NULL;
    }
    char * data_text = _item_label_recurrence_get(data);
    Evas_Object *content = _content_get(data, obj, label_recurrence, data_text);
    if(data_text)
    {
        free(data_text);
    }
    return content;
}

static Evas_Object *_item_content_priority_get(void *data, Evas_Object *obj, const char *part)
{
    if(strcmp(part, "elm.icon") != 0)
    {
        return NULL;
    }
    char *data_text = _item_label_priority_get(data);
    Evas_Object *content = _content_get(data, obj, label_priority, data_text);
    if(data_text)
    {
        free(data_text);
    }
    return content;
}

static Evas_Object *_item_content_sensivity_get(void *data, Evas_Object *obj, const char *part)
{
    if(strcmp(part, "elm.icon") != 0)
    {
        return NULL;
    }
    char * data_text = _item_label_sensivity_get(data);
    Evas_Object *content = _content_get(data, obj, label_sensivity, data_text);
    if(data_text)
    {
        free(data_text);
    }
    return content;
}

static Evas_Object *_item_content_status_get(void *data, Evas_Object *obj, const char *part)
{
    if(strcmp(part, "elm.icon") != 0)
    {
        return NULL;
    }
    char * data_text = _item_label_status_get(data);
    Evas_Object *content = _content_get(data, obj, label_status, data_text);
    if(data_text)
    {
        free(data_text);
    }
    return content;
}

static void _genlist_items_append(details_view_data *data)
{
    static Elm_Genlist_Item_Class itc_title =
    {
        .item_style = "1icon",
        .func.content_get = _item_content_get,
    };

    static Elm_Genlist_Item_Class itc_start =
    {
        .item_style = "1icon",
        .func.content_get = _item_content_start_get,
    };

    static Elm_Genlist_Item_Class itc_due =
    {
        .item_style = "1icon",
        .func.content_get = _item_content_due_get,
    };

    static Elm_Genlist_Item_Class itc_timezone =
    {
        .item_style = "1icon",
        .func.content_get = _item_content_timezone_get,
    };

    static Elm_Genlist_Item_Class itc_location =
    {
        .item_style = "1icon",
        .func.content_get = _item_content_location_get,
    };

    static Elm_Genlist_Item_Class itc_description =
    {
        .item_style = "1icon",
        .func.content_get = _item_content_description_get,
    };

    static Elm_Genlist_Item_Class itc_reminder =
    {
        .item_style = "1icon",
        .func.content_get = _item_content_reminder_get,
    };

    static Elm_Genlist_Item_Class itc_recurrence =
    {
        .item_style = "1icon",
        .func.content_get = _item_content_recurrence_get,
    };

    static Elm_Genlist_Item_Class itc_priority =
    {
        .item_style = "1icon",
        .func.content_get = _item_content_priority_get,
    };

    static Elm_Genlist_Item_Class itc_sensivity =
    {
        .item_style = "1icon",
        .func.content_get = _item_content_sensivity_get,
    };

    static Elm_Genlist_Item_Class itc_status =
    {
        .item_style = "1icon",
        .func.content_get = _item_content_status_get,
    };

    static Elm_Genlist_Item_Class itc_seperator =
    {
        .item_style = "separator",
        .func.text_get = NULL,
        .func.content_get = NULL,
        .func.state_get = NULL,
        .func.del = NULL,
    };

    elm_genlist_item_append(data->gl_controls, &itc_title,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_seperator,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_start,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_due,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_seperator,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_timezone,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_location,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_description,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_seperator,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_reminder,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_recurrence,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_seperator,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_priority,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_sensivity,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);

    elm_genlist_item_append(data->gl_controls, &itc_status,
                            data, NULL,
                            ELM_GENLIST_ITEM_NONE,
                            NULL, data);
}

static void _genlist_update(details_view_data *view_data)
{
    RETM_IF(!view_data, "Passed NULL input data");
    calendar_error_e error = CALENDAR_ERROR_NONE;
    calendar_connect();
    error = calendar_db_get_record(_calendar_event._uri, view_data->event_record_index, &view_data->event_record);
    if(CALENDAR_ERROR_NONE == error)
    {
        elm_genlist_realized_items_update(view_data->gl_controls);
    }
    calendar_disconnect();
}

static void _delete_event(calendar_record_h record)
{
    calendar_error_e error = calendar_connect();
    if(error == CALENDAR_ERROR_NONE)
    {
    int index = 0;
        error = calendar_record_get_int(record, _calendar_event.id, &index);
        if(error == CALENDAR_ERROR_NONE)
        {
            error = calendar_db_delete_record(_calendar_event._uri, index);
            WARN_IF(error != CALENDAR_ERROR_NONE, "calendar_db_delete_record() is failed(%x)", error);
        }
        else
        {
            WARN("getting event id is failed(%x)", error);
        }
        calendar_disconnect();
    }
}

static void _popup_ok_button_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    details_view_data *app = (details_view_data *)data;

    _delete_event(app->event_record);

    evas_object_del(app->popup);
    app->popup = NULL;
    elm_naviframe_item_pop(app->navi);
}

static void _popup_cancel_button_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    details_view_data *app = (details_view_data *)data;

    evas_object_del(app->popup);
    app->popup = NULL;
}

static void _create_confirmation_popup(details_view_data *app)
{
    RETM_IF(!app, "Passed NULL input data");

    Evas_Object *popup = elm_popup_add(app->navi);

    Evas_Object *popup_content = elm_label_add(popup);
    elm_object_text_set(popup_content, label_del_confirm);
    elm_object_content_set(popup, popup_content);

    Evas_Object *cancel_button = elm_button_add(popup);
    elm_object_text_set(cancel_button, button_cancel);
    elm_object_part_content_set(popup, "button1", cancel_button);
    evas_object_smart_callback_add(cancel_button, "clicked", _popup_cancel_button_cb, app);

    Evas_Object *ok_button = elm_button_add(popup);
    elm_object_text_set(ok_button, button_ok);
    elm_object_part_content_set(popup, "button2", ok_button);
    evas_object_smart_callback_add(ok_button, "clicked", _popup_ok_button_cb, app);

    app->popup = popup;
    evas_object_show(popup);
}

static void _delete_button_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    details_view_data *app = (details_view_data *)data;

    _create_confirmation_popup(app);
}

static void _edit_view_layout_del_cb(void *data, Evas *evas, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    details_view_data *view_data = (details_view_data *)data;
    _genlist_update(view_data);
}

static void edit_button_cb(void *data, Evas_Object *obj, void *event_info)
{
    RETM_IF(!data, "Passed NULL input data");
    details_view_data *view_data = (details_view_data *)data;

    Evas_Object *layout = open_edit_view(view_data->win, view_data->navi, view_data->event_record_index);
    evas_object_event_callback_add(layout, EVAS_CALLBACK_DEL, _edit_view_layout_del_cb, data);
}


static bool close_active_popup(details_view_data *view_data)
{
    RETVM_IF(!view_data, false, "view_data is NULL");

    if (view_data->popup)
    {
        evas_object_del(view_data->popup);
        view_data->popup = NULL;
        return true;
    }
    return false;
}


static Eina_Bool _details_view_navi_pop_cb(void *data, Elm_Object_Item *it)
{
    RETVM_IF(!data, EINA_FALSE, "Passed NULL input data");
    details_view_data *app = (details_view_data *)data;

    if (close_active_popup(app))
    {
        return EINA_FALSE;
    }
    else
    {
        evas_object_del(app->gl_controls);
        app->gl_controls = NULL;
        return EINA_TRUE;
    }
}

Evas_Object *details_view_add(window_obj *win, Evas_Object *parent, int index)
{
    RETVM_IF(!win || !parent, NULL, "Passed NULL input data");

    details_view_data *data = calloc(1, sizeof(details_view_data));
    RETVM_IF(!data, NULL, "Cannot allocate memory");

    data->win = win;
    data->navi = parent;
    data->event_record_index = index;
    calendar_connect();

    if(CALENDAR_ERROR_NONE != calendar_db_get_record(_calendar_event._uri, index, &data->event_record))
    {
        calendar_disconnect();
        free(data);
        return NULL;
    }

    size_t alarm_count = 0;
    calendar_record_get_child_record_count(data->event_record, _calendar_event.calendar_alarm, &alarm_count);

    if (alarm_count > 0)
    {
        calendar_record_get_child_record_at_p(data->event_record, _calendar_event.calendar_alarm, 0, &data->alarm_record);
    }

    data->layout = ui_utils_layout_add(data->navi, details_view_destroy_cb, data);
    evas_object_size_hint_weight_set(data->layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
    evas_object_size_hint_align_set(data->layout, EVAS_HINT_FILL, EVAS_HINT_FILL);
    elm_layout_file_set(data->layout, GET_PATH(EDJ_CREATEVIEW), "create_view_layout");

    /*Header for naviframe*/
    data->navi_item = elm_naviframe_item_push(data->navi, "Details", NULL, NULL, data->layout, NULL);
    elm_naviframe_item_pop_cb_set(data->navi_item, _details_view_navi_pop_cb, data);
    evas_object_show(data->layout);

    /*Footer for naviframe*/
    Evas_Object * toolbar = ui_utils_toolbar_add(data->navi, data->navi_item);
    elm_toolbar_select_mode_set(toolbar, ELM_OBJECT_SELECT_MODE_NONE);

    data->edit_button = elm_toolbar_item_append(toolbar, NULL, button_edit, edit_button_cb, data);
    data->delete_button = elm_toolbar_item_append(toolbar, NULL, button_delete, _delete_button_cb, data);

    int calendar_book_id = 0;
    calendar_error_e error = calendar_record_get_int(data->event_record, _calendar_event.calendar_book_id, &calendar_book_id);
    WARN_IF(error != CALENDAR_ERROR_NONE, "calendar_record_get_int is failed(%x)", error);

    if( calendar_book_id != DEFAULT_EVENT_CALENDAR_BOOK_ID
        && calendar_book_id != DEFAULT_TODO_CALENDAR_BOOK_ID
        && calendar_book_id != DEFAULT_BIRTHDAY_CALENDAR_BOOK_ID )
    {
        elm_object_item_disabled_set(data->edit_button, EINA_TRUE);
        elm_object_item_disabled_set(data->delete_button, EINA_TRUE);
    }

    /*Create genlist for Events control*/
    _genlist_add(data);

    _genlist_items_append(data);
    calendar_disconnect();

    return data->layout;
}
